"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDownload = exports.downloadFile = exports.getDownloadMetadata = exports.testFile = exports.checkDestination = exports.downloadUrl = void 0;
require("cross-fetch/polyfill");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const config_js_1 = require("./config.js");
exports.downloadUrl = (baseUrl, datasetId, tag) => tag
    ? `${baseUrl}crn/datasets/${datasetId}/snapshots/${tag}/download`
    : `${baseUrl}crn/datasets/${datasetId}/download`;
exports.checkDestination = destination => {
    if (fs_1.default.existsSync(destination)) {
        // Exists, check if directory
        if (!fs_1.default.lstatSync(destination).isDirectory()) {
            throw new Error(`Target file "${destination}" must be a directory`);
        }
    }
    else {
        // Doesn't exist, make directory
        fs_1.default.mkdirSync(destination);
    }
};
/**
 * Test if a file exists and matches correct size
 * @param {string} destination
 * @param {string} filename
 * @param {number} size
 */
exports.testFile = (destination, filename, size) => {
    const fullPath = path_1.default.join(destination, filename);
    if (fs_1.default.existsSync(fullPath)) {
        return fs_1.default.lstatSync(fullPath).size !== size;
    }
    else {
        return true;
    }
};
const getFetchHeaders = () => ({
    Authorization: `Bearer ${config_js_1.getToken()}`,
});
const handleFetchReject = err => {
    console.error('Error starting download - please check your connection or try again later');
    console.dir(err);
};
/**
 * Obtain the listing of files to download
 * @param {string} datasetId
 * @param {string} tag
 */
exports.getDownloadMetadata = async (datasetId, tag) => {
    try {
        const response = await fetch(exports.downloadUrl(config_js_1.getUrl(), datasetId, tag), {
            headers: getFetchHeaders(),
        });
        if (response.status === 200) {
            const body = await response.json();
            return body;
        }
        else {
            console.error(`Error ${response.status} fetching download metadata - ${response.statusText}`);
        }
    }
    catch (err) {
        handleFetchReject(err);
    }
};
/**
 * Download one file from a remote URL
 * @param {string} destination Destination directory path
 * @param {string} filename
 * @param {string} fileUrl URL to download from
 * @param {object} apmTransaction Active APM transaction (optional)
 */
exports.downloadFile = async (destination, filename, fileUrl, apmTransaction) => {
    try {
        const fullPath = path_1.default.join(destination, filename);
        // Create any needed parent dirs
        mkdirp_1.default.sync(path_1.default.dirname(fullPath));
        const writeStream = fs_1.default.createWriteStream(fullPath);
        try {
            const response = await fetch(fileUrl, {
                headers: getFetchHeaders(),
            });
            // @ts-expect-error
            const stream = await response.readable();
            if (response.status === 200) {
                // Setup end/error handler with Promise interface
                const responsePromise = new Promise((resolve, reject) => {
                    stream.on('end', () => resolve());
                    stream.on('error', err => {
                        if (apmTransaction)
                            apmTransaction.captureError(err);
                        reject(err);
                    });
                });
                // Start piping data
                stream.pipe(writeStream);
                return responsePromise;
            }
            else {
                console.error(`Error ${response.status} fetching "${filename}" - ${response.statusText}`);
            }
        }
        catch (err) {
            handleFetchReject(err);
        }
    }
    catch (err) {
        if (apmTransaction)
            apmTransaction.captureError(err);
        throw err;
    }
};
exports.getDownload = (destination, datasetId, tag, apmTransaction) => {
    const apmSetup = apmTransaction.startSpan('getDownloadMetadata');
    return exports.getDownloadMetadata(datasetId, tag).then(async (body) => {
        apmTransaction.addLabels({ datasetId, tag });
        exports.checkDestination(destination);
        apmSetup.end();
        for (const file of body.files) {
            if (exports.testFile(destination, file.filename, file.size)) {
                // Now actually download
                const apmDownload = apmTransaction.startSpan(`download ${file.filename}:${file.size}`);
                // eslint-disable-next-line no-console
                console.log(`Downloading "${file.filename}" - size ${file.size} bytes`);
                await exports.downloadFile(destination, file.filename, file.urls.pop(), apmTransaction);
                if (apmDownload)
                    apmDownload.end();
            }
            else {
                // eslint-disable-next-line no-console
                console.log(`Skipping present file "${file.filename}"`);
            }
        }
    });
};
