"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.gitAnnexRemote = exports.response = exports.handleGitAnnexMessage = void 0;
const readline_1 = __importDefault(require("readline"));
const events_1 = require("events");
const gitCredential_js_1 = require("./gitCredential.js");
const transferKey_js_1 = require("./transferKey.js");
const GIT_ANNEX_VERSION = 'VERSION 1';
async function handleGitAnnexMessage(line, state) {
    if (line.startsWith('EXTENSIONS')) {
        return 'EXTENSIONS';
    }
    else if (line.startsWith('PREPARE')) {
        // Ask for configuration to validate
        return 'GETCONFIG url';
    }
    else if (line.startsWith('VALUE ')) {
        // Check if VALUE is configured already
        if (state.url) {
            return 'PREPARE-SUCCESS';
        }
        else {
            return 'PREPARE-FAILURE url must be configured when running initremote or enableremote';
        }
    }
    else if (line.startsWith('TRANSFER STORE')) {
        const [, , key, file] = line.split(' ', 4);
        if (await transferKey_js_1.storeKey(state, key, file)) {
            return `TRANSFER-SUCCESS STORE ${key}`;
        }
        else {
            return `TRANSFER-FAILURE STORE ${key}`;
        }
    }
    else if (line.startsWith('TRANSFER RETRIEVE')) {
        const [, , key, file] = line.split(' ', 4);
        if (await transferKey_js_1.retrieveKey(state, key, file)) {
            return `TRANSFER-SUCCESS RETRIEVE ${key}`;
        }
        else {
            return `TRANSFER-FAILURE RETRIEVE ${key}`;
        }
    }
    else if (line.startsWith('CHECKPRESENT')) {
        const key = line.split('CHECKPRESENT ', 2)[1];
        if (await transferKey_js_1.checkKey(state, key)) {
            return `CHECKPRESENT-SUCCESS ${key}`;
        }
        else {
            return `CHECKPRESENT-FAILURE ${key}`;
        }
    }
    else if (line.startsWith('INITREMOTE')) {
        // No init steps are required - always succeed
        return 'INITREMOTE-SUCCESS';
    }
    else if (line.startsWith('GETAVAILABILITY')) {
        return 'AVAILABILITY GLOBAL';
    }
    else if (line.startsWith('REMOVE')) {
        const key = line.split('REMOVE ', 2)[1];
        if (await transferKey_js_1.removeKey(state, key)) {
            return `REMOVE-SUCCESS ${key}`;
        }
        else {
            return `REMOVE-FAILURE ${key}`;
        }
    }
    else {
        return 'UNSUPPORTED-REQUEST';
    }
}
exports.handleGitAnnexMessage = handleGitAnnexMessage;
/**
 * Stateful response handling for git annex protocol
 */
exports.response = () => {
    const state = {};
    return async (line) => {
        if (line.startsWith('VALUE ')) {
            try {
                const url = line.split('VALUE ')[1];
                // Obtain the filename (no extensions) in url value
                const datasetId = url.substring(url.lastIndexOf('/') + 1, url.length);
                state.url = url;
                state.token = await gitCredential_js_1.getRepoToken(datasetId);
            }
            catch (err) {
                state.url = undefined;
                state.token = undefined;
            }
        }
        console.log(await handleGitAnnexMessage(line, state));
    };
};
async function gitAnnexRemote() {
    try {
        const rl = readline_1.default.createInterface({
            input: process.stdin,
        });
        console.log(GIT_ANNEX_VERSION);
        rl.on('line', void exports.response());
        await events_1.once(rl, 'close');
        process.exit(0);
    }
    catch (err) {
        console.error(err);
    }
}
exports.gitAnnexRemote = gitAnnexRemote;
