"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeKey = exports.checkKey = exports.retrieveKey = exports.storeKey = exports.keyRequest = void 0;
const fs_1 = __importDefault(require("fs"));
const fetch_h2_1 = require("fetch-h2");
const events_1 = require("events");
/**
 * Create a Request object for this url and key
 * @param {object} state
 * @param {string} state.url Base URL
 * @param {string} state.token Basic auth token for repos
 * @param {string} key git-annex key
 * @param {object} options fetch options
 * @returns {Request} Configured fetch Request object
 */
function keyRequest(state, key, options) {
    const headers = new fetch_h2_1.Headers();
    headers.set('Authorization', 'Basic ' + Buffer.from(`openneuro-cli:${state.token}`).toString('base64'));
    const requestUrl = `${state.url}/annex/${key}`;
    return new fetch_h2_1.Request(requestUrl, { headers, ...options });
}
exports.keyRequest = keyRequest;
/**
 * Call POST to upload a key to a remote
 * @param {object} state
 * @param {string} state.url Base URL
 * @param {string} state.token Basic auth token for repos
 * @param {string} key Git-annex key
 * @param {string} file File path
 */
async function storeKey(state, key, file) {
    const body = fs_1.default.createReadStream(file);
    const request = keyRequest(state, key, { body, method: 'POST' });
    const response = await fetch_h2_1.fetch(request);
    if (response.status === 200) {
        return true;
    }
    else {
        return false;
    }
}
exports.storeKey = storeKey;
/**
 * Call GET to download a key from a remote
 * @param {object} state
 * @param {string} state.url Base URL
 * @param {string} state.token Basic auth token for repos
 * @param {string} key Git-annex key
 * @param {string} file File path
 */
async function retrieveKey(state, key, file) {
    try {
        const request = keyRequest(state, key, { method: 'GET' });
        const response = await fetch_h2_1.fetch(request);
        if (response.status === 200) {
            const writable = fs_1.default.createWriteStream(file);
            const readable = await response.readable();
            readable.pipe(writable);
            await events_1.once(readable, 'close');
            return true;
        }
        else {
            return false;
        }
    }
    catch (err) {
        console.error(err);
        return false;
    }
}
exports.retrieveKey = retrieveKey;
/**
 * Call HEAD to check if key exists on remote
 * @param {object} state
 * @param {string} state.url Base URL
 * @param {string} state.token Basic auth token for repos
 * @param {string} key
 * @returns {Promise<boolean>} True or false if key exists
 */
async function checkKey(state, key) {
    const request = keyRequest(state, key, { method: 'HEAD' });
    const response = await fetch_h2_1.fetch(request);
    if (response.status === 200) {
        return true;
    }
    else {
        return false;
    }
}
exports.checkKey = checkKey;
/**
 * Call DELETE to remove a key from the remote
 * @param {object} state
 * @param {string} state.url Base URL
 * @param {string} state.token Basic auth token for repos
 * @param {string} key
 * @returns {Promise<boolean>} True or false if key exists
 */
async function removeKey(state, key) {
    const request = keyRequest(state, key, { method: 'DELETE' });
    const response = await fetch_h2_1.fetch(request);
    if (response.status === 204) {
        return true;
    }
    else {
        return false;
    }
}
exports.removeKey = removeKey;
