"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.finishUpload = exports.uploadFiles = exports.prepareUpload = exports.validation = void 0;
const fetch_h2_1 = require("fetch-h2");
const cli_progress_1 = __importDefault(require("cli-progress"));
const path_1 = __importDefault(require("path"));
const inquirer_1 = __importDefault(require("inquirer"));
const fs_1 = require("fs");
const openneuro_client_1 = require("openneuro-client");
const bids_validator_1 = __importDefault(require("bids-validator"));
const files_1 = require("./files");
const config_1 = require("./config");
const consoleFormat_1 = __importDefault(require("bids-validator/utils/consoleFormat"));
/**
 * BIDS validator promise wrapper
 *
 * @param {string} dir directory to validate
 * @param {Object} options validate.BIDS options
 */
const validatePromise = (dir, options = {}) => {
    return new Promise((resolve, reject) => {
        bids_validator_1.default.BIDS(dir, options, ({ errors, warnings }, summary) => {
            if (errors.length + warnings.length === 0) {
                resolve({ summary });
            }
            else {
                reject(consoleFormat_1.default.issues({ errors, warnings }));
            }
        });
    });
};
const fatalError = (err, apmSpan) => {
    // eslint-disable-next-line no-console
    console.error(err);
    apmSpan.end();
    process.exit(1);
};
/**
 * Runs validation, logs summary or exits if an error is encountered
 * @param {string} dir Directory to validate
 * @param {object} validatorOptions Options passed to the validator
 * @param {object} apmTransaction Elastic APM Transaction object
 */
exports.validation = (dir, validatorOptions, apmTransaction) => {
    const apmValidatePromiseSpan = apmTransaction && apmTransaction.startSpan('validatePromise');
    return validatePromise(dir, validatorOptions)
        .then(function ({ summary }) {
        // eslint-disable-next-line no-console
        console.log(consoleFormat_1.default.summary(summary));
        apmValidatePromiseSpan.end();
    })
        .catch(err => fatalError(err, apmValidatePromiseSpan));
};
/**
 * Calculate the diff and prepare an upload
 *
 * @param {Object} client - Initialized Apollo client to call prepareUpload and finishUpload
 * @param {string} dir - Directory to upload
 * @param {Object} options Query parameters for prepareUpload mutation
 * @param {string} options.datasetId Accession number
 * @param {Array<object>} options.remoteFiles An array of files available in HEAD, matching files are skipped
 * @returns {Promise<Object|void>} prepareUpload mutation fields {id, token, files}
 */
exports.prepareUpload = async (client, dir, { datasetId, remoteFiles }) => {
    const files = [];
    for await (const f of files_1.getFiles(dir)) {
        const rel = path_1.default.relative(dir, f);
        const remote = remoteFiles.find(remote => remote.filename === rel);
        const { size } = await fs_1.promises.stat(f);
        if (remote) {
            if (remote.size !== size) {
                files.push({ filename: rel, path: f, size });
            }
        }
        else {
            files.push({ filename: rel, path: f, size });
        }
    }
    console.log('=======================================================================');
    if (files.length < 12) {
        console.log('Files to be uploaded:');
        for (const f of files) {
            console.log(`${f.filename} - ${files_1.bytesToSize(f.size)}`);
        }
    }
    else {
        const totalSize = openneuro_client_1.uploads.uploadSize(files);
        console.log(`${files.length} files to be uploaded with a total size of ${files_1.bytesToSize(totalSize)}`);
    }
    const answer = await inquirer_1.default.prompt({
        type: 'confirm',
        name: 'start',
        message: 'Begin upload?',
        default: true,
    });
    if (answer.start) {
        console.log('=======================================================================');
        // Filter out local paths in mutation
        const mutationFiles = files.map(f => ({
            filename: f.filename,
            size: f.size,
        }));
        const { data } = await client.mutate({
            mutation: openneuro_client_1.uploads.prepareUpload,
            variables: {
                datasetId,
                uploadId: openneuro_client_1.uploads.hashFileList(datasetId, mutationFiles),
            },
        });
        const id = data.prepareUpload.id;
        // eslint-disable-next-line no-console
        console.log(`Starting a new upload (${id}) to dataset: '${datasetId}'`);
        return {
            id,
            datasetId: data.prepareUpload.datasetId,
            token: data.prepareUpload.token,
            files,
            endpoint: data.prepareUpload.endpoint,
        };
    }
};
exports.uploadFiles = async ({ id, datasetId, token, files, endpoint, }) => {
    const uploadProgress = new cli_progress_1.default.SingleBar({
        format: datasetId + ' [{bar}] {percentage}% | ETA: {eta}s | {value}/{total}',
        clearOnComplete: false,
        hideCursor: true,
        etaBuffer: Math.round(files.length / 10) + 1,
    });
    uploadProgress.start(files.length, 0, {
        speed: 'N/A',
    });
    const rootUrl = config_1.getUrl();
    const controller = new fetch_h2_1.AbortController();
    const requests = files.map(file => {
        // http://localhost:9876/uploads/0/ds001024/0de963b9-1a2a-4bcc-af3c-fef0345780b0/dataset_description.json
        const encodedFilePath = openneuro_client_1.uploads.encodeFilePath(file.filename);
        const fileStream = fs_1.createReadStream(file.path);
        fileStream.on('error', err => {
            console.error(err);
            controller.abort();
        });
        fileStream.on('close', () => {
            if (fileStream.bytesRead === 0) {
                uploadProgress.stop();
                console.error(`Warning: "${file.filename}" read zero bytes - check that this file is readable and try again`);
                controller.abort();
            }
        });
        return new fetch_h2_1.Request(`${rootUrl}uploads/${endpoint}/${datasetId}/${id}/${encodedFilePath}`, {
            method: 'POST',
            headers: {
                Authorization: `Bearer ${token}`,
            },
            body: fileStream,
            // @ts-expect-error - this is missing in the upstream type
            signal: controller.signal,
        });
    });
    await openneuro_client_1.uploads.uploadParallel(requests, openneuro_client_1.uploads.uploadSize(files), uploadProgress, fetch_h2_1.fetch);
    uploadProgress.stop();
};
exports.finishUpload = (client, uploadId) => {
    return client.mutate({
        mutation: openneuro_client_1.uploads.finishUpload,
        variables: { uploadId: uploadId },
    });
};
